
open Ast
open Format

(* Excepção levantada para assinalar um erro durante a interpretação *)

exception Error of string
let error s = raise (Error s)

(* Os valores de Mini-Python

   - uma diferença notável com Python : utilizamos o tipo int enquanto
     os inteiros de Python são de precisão arbitrária ; poderiamos ter
     escolhido o tipo disponível no módulo Big_int d'OCaml, mas preferimos
     aqui a solução "fácil"

   - o que Python designa por lista é na realidade um vector que se pode
     redimensionar ; no fragmento considerado aqui, não existe a possibilidade
     de modificação do comprimento, assim um simples vector OCaml chega *)
type value =
  | Vnone
  | Vbool of bool
  | Vint of int
  | Vstring of string
  | Vlist of value array

(* Vizualização de um valor na saída standard *)
let rec print_value = function
  | Vnone -> printf "None"
  | Vbool true -> printf "True"
  | Vbool false -> printf "False"
  | Vint n -> printf "%d" n
  | Vstring s -> printf "%s" s
  | Vlist a ->
    let n = Array.length a in
    printf "[";
    for i = 0 to n-1 do print_value a.(i); if i < n-1 then printf ", " done;
    printf "]"

(* Interpretação booleana de um valor

   Em Python, qualquer valor pode ser utilizado como um boleano : None,
   a lista vazia, a string vazia e o inteiro 0 são considerados como
   falso/False e qualquer outro valor é considerado verdade/True *)

let is_false v = assert false (* por completar (questão 2) *)

let is_true v = assert false (* por completar (questão 2) *)

(* Só consideramos aqui as funções globais *)

let functions = (Hashtbl.create 16 : (string, ident list * stmt) Hashtbl.t)

(* A instrução 'return' de Python é interpretada com a ajuda de uma excepção *)

exception Return of value

(* As variáveis locais (parâmetros de funções e variáveis introduzidos
   pelas atribuições) são arquivadas numa tabela de dispersão (hashtable)
   passada como argumento nas funções seguintes com o nome 'ctx' *)

type ctx = (string, value) Hashtbl.t

(* Interpretação de uma expressão (retorna um valor) *)

let rec expr ctx = function
  | Ecst Cnone ->
      Vnone
  | Ecst (Cstring s) ->
      Vstring s
  (* aritmética *)
  | Ecst (Cint n) ->
      assert false  (* por completar (questão 1) *)
  | Ebinop (Badd | Bsub | Bmul | Bdiv | Bmod |
            Beq | Bneq | Blt | Ble | Bgt | Bge as op, e1, e2) ->
      let v1 = expr ctx e1 in
      let v2 = expr ctx e2 in
      begin match op, v1, v2 with
        | Badd, Vint n1, Vint n2 -> assert false (* por completar (questão 1) *)
        | Bsub, Vint n1, Vint n2 -> assert false (* por completar (questão 1) *)
        | Bmul, Vint n1, Vint n2 -> assert false (* por completar (questão 1) *)
        | Bdiv, Vint n1, Vint n2 -> assert false (* por completar (questão 1) *)
        | Bmod, Vint n1, Vint n2 -> assert false (* por completar (questão 1) *)
        | Beq, _, _  -> assert false (* por completar (questão 2) *)
        | Bneq, _, _ -> assert false (* por completar (questão 2) *)
        | Blt, _, _  -> assert false (* por completar (questão 2) *)
        | Ble, _, _  -> assert false (* por completar (questão 2) *)
        | Bgt, _, _  -> assert false (* por completar (questão 2) *)
        | Bge, _, _  -> assert false (* por completar (questão 2) *)
        | Badd, Vstring s1, Vstring s2 ->
            assert false (* por completar (questão 3) *)
        | Badd, Vlist l1, Vlist l2 ->
            assert false (* por completar (questão 5) *)
        | _ -> error "unsupported operand types"
      end
  | Eunop (Uneg, e1) ->
      assert false (* por completar (questão 1) *)
  (* booleanos *)
  | Ecst (Cbool b) ->
      assert false (* por completar (questão 2) *)
  | Ebinop (Band, e1, e2) ->
      assert false (* por completar (questão 2) *)
  | Ebinop (Bor, e1, e2) ->
      assert false (* por completar (questão 2) *)
  | Eunop (Unot, e1) ->
      assert false (* por completar (questão 2) *)
  | Eident id ->
      assert false (* por completar (questão 3) *)
  (* chamadas de função *)
  | Ecall ("len", [e1]) ->
      assert false (* por completar (questão 5) *)
  | Ecall ("list", [Ecall ("range", [e1])]) ->
      assert false (* por completar (questão 5) *)
  | Ecall (f, el) ->
      assert false (* por completar (questão 4) *)
  | Elist el ->
      assert false (* por completar (questão 5) *)
  | Eget (e1, e2) ->
      assert false (* por completar (questão 5) *)

(* interpretação de uma instrução ; não retorna nada *)

and stmt ctx = function
  | Seval e ->
      ignore (expr ctx e)
  | Sprint e ->
      print_value (expr ctx e); printf "@."
  | Sblock bl ->
      block ctx bl
  | Sif (e, s1, s2) ->
      assert false (* por completar (questão 2) *)
  | Sassign (id, e1) ->
      assert false (* por completar (questão 3) *)
  | Sreturn e ->
      assert false (* por completar (questão 4) *)
  | Sfor (x, e, s) ->
      assert false (* por completar (questão 5) *)
  | Sset (e1, e2, e3) ->
      assert false (* por completar (questão 5) *)

(* interpretação de um bloco i.e. de uma sequência de instruções *)

and block ctx = function
  | [] -> ()
  | s :: sl -> stmt ctx s; block ctx sl

(* interpretação de um ficheiro
   - dl é uma lista de definições de funcões (ver  Ast.def)
   - s é uma instrução que representa as instruções globais
*)

let file (dl, s) =
  (* por completar (questão 4) *)
  stmt (Hashtbl.create 16) s
